import { getCurrentTimeOfDay, weathers } from "./world";
export const rarityPrefixes = ["iron", "golden", "diamond"];
export const vanillaFishes = [
    "minecraft:cod",
    "minecraft:salmon",
    "minecraft:tropical_fish",
];
export const garbageItems = [
    {
        id: "minecraft:stick",
        amount: { min: 1, max: 4 }
    },
    {
        id: "minecraft:iron_nugget",
        amount: { min: 1, max: 2 }
    },
    {
        id: "minecraft:gold_nugget",
        amount: { min: 1, max: 2 }
    },
    {
        id: "minecraft:kelp",
        amount: { min: 1, max: 3 }
    },
    {
        id: "minecraft:seagrass",
        amount: { min: 1, max: 3 }
    },
    {
        id: "aquarium:old_paper",
        amount: { min: 1, max: 2 }
    }
];
export const fishingLoot = {
    eel: {
        id: "aquarium:eel",
        weightRange: [1, 3],
        weather: ["clear", "rain"],
        time: ["sunset", "night", "midnight"],
        biomes: {
            swampland: { probability: 0.2 },
            river: { probability: 0.18 },
            mangrove_swamp: { probability: 0.15 },
            deep_dark: { probability: 0.1 },
        }
    },
    jellyfish: {
        id: "aquarium:jellyfish",
        weightRange: [0.5, 5],
        weather: ["clear", "rain"],
        time: ["night", "midnight"],
        biomes: {
            warm_ocean: { probability: 0.18 },
            lukewarm_ocean: { probability: 0.15 },
            deep_warm_ocean: { probability: 0.1 },
        }
    },
    crab: {
        id: "aquarium:crab",
        weightRange: [0.3, 2],
        weather: ["clear", "rain"],
        time: ["sunrise", "day", "sunset"],
        biomes: {
            beach: { probability: 0.25 },
            river: { probability: 0.15 },
            swampland: { probability: 0.2 },
            mangrove_swamp: { probability: 0.22 },
            mushroom_island_shore: { probability: 0.1 },
        }
    },
    swordfish: {
        id: "aquarium:swordfish",
        weightRange: [30, 50],
        weather: ["clear"],
        time: ["noon", "night"],
        biomes: {
            deep_ocean: { probability: 0.08 },
            deep_lukewarm_ocean: { probability: 0.05 },
            deep_warm_ocean: { probability: 0.04 },
        }
    },
    tuna: {
        id: "aquarium:tuna",
        weightRange: [10, 17],
        weather: ["clear"],
        time: ["day", "noon"],
        biomes: {
            deep_ocean: { probability: 0.15 },
            deep_lukewarm_ocean: { probability: 0.12 },
            deep_warm_ocean: { probability: 0.1 },
            warm_ocean: { probability: 0.05 },
        }
    },
    catfish: {
        id: "aquarium:catfish",
        weightRange: [2, 20],
        weather: ["clear", "rain", "thunder"],
        time: ["sunset", "night"],
        biomes: {
            river: { probability: 0.25 },
            swamp: { probability: 0.22 },
            taiga: { probability: 0.15 },
            cold_river: { probability: 0.1 },
            lush_caves: { probability: 0.1 },
        }
    },
    goldfish: {
        id: "aquarium:goldfish",
        weightRange: [0.1, 0.4],
        weather: ["clear"],
        time: ["day", "sunrise"],
        biomes: {
            river: { probability: 0.3 },
            forest: { probability: 0.25 },
            plains: { probability: 0.25 },
            flower_forest: { probability: 0.2 },
            lush_caves: { probability: 0.15 },
            cherry_grove: { probability: 0.15 },
        }
    },
    seahorse: {
        id: "aquarium:seahorse",
        weightRange: [0.02, 0.1],
        weather: ["clear"],
        time: ["day", "sunrise"],
        biomes: {
            warm_ocean: { probability: 0.1 },
            lukewarm_ocean: { probability: 0.06 },
            deep_warm_ocean: { probability: 0.03 },
        }
    },
    anchovy: {
        id: "aquarium:anchovy",
        weightRange: [0.015, 0.05],
        weather: ["clear", "rain"],
        time: ["sunrise", "day", "sunset"],
        biomes: {
            lukewarm_ocean: { probability: 0.22 },
            warm_ocean: { probability: 0.18 },
            beach: { probability: 0.15 },
            river: { probability: 0.10 },
            deep_lukewarm_ocean: { probability: 0.08 },
            cold_ocean: { probability: 0.05 },
            mangrove_swamp: { probability: 0.12 },
        }
    },
    blobfish: {
        id: "aquarium:blobfish",
        weightRange: [1, 2],
        weather: ["clear", "rain"],
        time: ["midnight", "night"],
        biomes: {
            deep_ocean: { probability: 0.08 },
            deep_frozen_ocean: { probability: 0.05 },
            deep_cold_ocean: { probability: 0.06 },
        }
    },
    frilled_shark: {
        id: "aquarium:frilled_shark",
        weightRange: [20, 30],
        weather: ["clear"],
        time: ["night", "midnight"],
        biomes: {
            deep_ocean: { probability: 0.04 },
            deep_dark: { probability: 0.03 },
            deep_frozen_ocean: { probability: 0.02 },
        }
    },
    astralish: {
        id: "aquarium:astralish",
        weightRange: [12, 23],
        weather: ["any"],
        time: ["any"],
        biomes: {
            the_end: { probability: 0.17 },
        }
    },
};
export function fishInBiome(biome, weather, tick) {
    const timeOfDay = getCurrentTimeOfDay(tick);
    const validItems = [];
    const rarityNumbers = rarityPrefixes.length;
    for (const key in fishingLoot) {
        const item = fishingLoot[key];
        const biomeData = item.biomes[biome] || item.biomes["any"];
        if (!biomeData)
            continue;
        const itemWeathers = item.weather;
        const itemTimes = item.time;
        const isWeatherValid = itemWeathers.includes("any") || weathers.some(w => itemWeathers.includes(w));
        const isTimeValid = itemTimes.includes("any") || itemTimes.includes(timeOfDay);
        if (!isTimeValid || !isWeatherValid)
            continue;
        validItems.push({
            ...item,
            key,
            probability: biomeData.probability,
        });
    }
    if (validItems.length === 0) {
        // console.warn(`No hay ítems válidos con esos requerimientos. ${biome.id}`);
        return null;
    }
    const totalProbability = validItems.reduce((sum, item) => sum + item.probability, 0);
    const rand = Math.random() * totalProbability;
    let cumulative = 0;
    let selectedItem = null;
    for (const item of validItems) {
        cumulative += item.probability;
        if (rand <= cumulative) {
            selectedItem = item;
            break;
        }
    }
    if (!selectedItem && validItems.length > 0) {
        selectedItem = validItems[0];
    }
    if (!selectedItem)
        return;
    // console.warn(`No selectedItem`);
    const [min, max] = selectedItem.weightRange;
    const range = max - min;
    const step = range / rarityNumbers;
    const rarityChances = generateRarityChances(rarityNumbers);
    const rarityRand = Math.random();
    let tier = 0;
    let cumulativeChance = 0;
    for (let i = 0; i < rarityNumbers; i++) {
        cumulativeChance += rarityChances[i];
        if (rarityRand <= cumulativeChance) {
            tier = i;
            break;
        }
    }
    const tierMin = min + tier * step;
    const tierMax = tierMin + step;
    const weight = parseFloat((Math.random() * (tierMax - tierMin) + tierMin).toFixed(2));
    const [namespace, baseId] = selectedItem.id.split(":");
    const newId = `${namespace}:${rarityPrefixes[tier]}_${baseId}`;
    return {
        id: newId,
        originalId: selectedItem.id,
        weight,
        rarity: rarityPrefixes[tier],
        biome,
        weatherUsed: weather,
        timeUsed: timeOfDay
    };
}
function generateRarityChances(n) {
    const base = 4;
    const weights = Array.from({ length: n }, (_, i) => 1 / Math.pow(base, i));
    const total = weights.reduce((sum, w) => sum + w, 0);
    return weights.map(w => w / total);
}
