import { BiomeTypes } from "@minecraft/server";
export const weathers = [
    "clear", "rain", "thunder"
];
export const timeRanges = {
    day: [1000, 6000],
    noon: [6000, 12000],
    sunset: [12000, 13000],
    night: [13000, 18000],
    midnight: [18000, 23000],
    sunrise: [[23000, 24000], [0, 1000]]
};
export function getCurrentTimeOfDay(tick) {
    for (const timeRange in timeRanges) {
        const range = timeRanges[timeRange];
        if (Array.isArray(range[0])) {
            for (const [start, end] of range) {
                if (tick >= start && tick < end)
                    return timeRange;
            }
        }
        else {
            const [start, end] = range;
            if (tick >= start && tick < end)
                return timeRange;
        }
    }
    return "unknown";
}
export function getBiome(location, dimension) {
    let closestBiome;
    for (const biome of BiomeTypes.getAll()) {
        const biomeLocation = dimension.findClosestBiome(location, biome, { boundingSize: { x: 64, y: 64, z: 64 } });
        if (biomeLocation) {
            const dx = biomeLocation.x - location.x;
            const dy = biomeLocation.y - location.y;
            const dz = biomeLocation.z - location.z;
            const distance = Math.sqrt(dx * dx + dy * dy + dz * dz);
            if (!closestBiome || distance < closestBiome.distance)
                closestBiome = { biome, distance };
        }
    }
    if (!closestBiome)
        throw new Error("Could not find biome within the given location");
    return closestBiome.biome;
}
