import { EquipmentSlot, GameMode, ItemStack, Player, system, world } from "@minecraft/server";
import { fishInBiome, garbageItems, vanillaFishes } from "lib/fishing";
import { decreaseItem, getEquipmentSlot } from "lib/item";
import { randomWholeNum } from "lib/maths";
import { getBiome } from "lib/world";
// world.afterEvents.playerPlaceBlock.subscribe(({ player }) => {
//   const tick = world.getTimeOfDay()
//   const weather = player.dimension.getWeather().toLowerCase()
//   const biome = getBiome(player.location, player.dimension).id.split(":")[1]
//   const fishing = fishInBiome(biome, weather, tick);
//   if (fishing === null) {
//     const item = new ItemStack("minecraft:cod")
//     player.getComponent("inventory").container.addItem(item)
//     return
//   }
//   const { id, weight, rarity } = fishing
//   if (rarity === "diamond") player.playSound("aquarium.effect.rare_item")
//   const item = new ItemStack(id)
//   item.setLore([
//     `§r§7Weight: ${weight}kg`,
//     `§r§7Rarity: ${rarity.charAt(0).toUpperCase() + rarity.slice(1)}`,
//   ]);
//   player.getComponent("inventory").container.addItem(item)
//   player.sendMessage(JSON.stringify(fishing))
// })
world.beforeEvents.entityRemove.subscribe(({ removedEntity: entity }) => {
    const item = entity.getComponent("item")?.itemStack;
    if (!item)
        return;
    const players = entity.dimension.getEntities({ maxDistance: 3, location: entity.location, type: "player" });
    for (const player of players) {
        if (player instanceof Player) {
            const inv = player.getComponent("inventory").container;
            system.runTimeout(() => {
                for (let i = 0; i < inv.size; i++) {
                    const item = inv.getItem(i);
                    if (!item || item.typeId !== "aquarium:random_fish")
                        continue;
                    const tick = world.getTimeOfDay();
                    const weather = player.dimension.getWeather().toLowerCase();
                    const biome = getBiome(player.location, player.dimension).id.split(":")[1];
                    const fishing = fishInBiome(biome, weather, tick);
                    // console.warn(getBiome(player.location, player.dimension).id)
                    if (fishing === null) {
                        const randomFish = vanillaFishes[Math.floor(Math.random() * vanillaFishes.length)];
                        inv.setItem(i, new ItemStack(randomFish));
                        return;
                    }
                    const fish = new ItemStack(fishing.id);
                    if (fishing.rarity === "diamond")
                        player.playSound("aquarium.effect.rare_item");
                    fish.setLore([
                        `§r§7Weight: ${fishing.weight}kg`,
                        `§r§7Rarity: ${fishing.rarity.charAt(0).toUpperCase() + fishing.rarity.slice(1)}`,
                    ]);
                    inv.setItem(i, fish);
                }
            });
        }
    }
});
world.afterEvents.itemCompleteUse.subscribe(({ source: player, itemStack: item }) => {
    if (item.typeId === "aquarium:garbage_bag") {
        const inv = player.getComponent("inventory").container;
        const mainhand = getEquipmentSlot(player, EquipmentSlot.Mainhand);
        const randomIndex = Math.floor(Math.random() * garbageItems.length);
        const randomitemData = garbageItems[randomIndex];
        const quantity = randomWholeNum(randomitemData.amount.min, randomitemData.amount.max);
        const randomItem = new ItemStack(randomitemData.id, quantity);
        if (player.getGameMode() !== GameMode.creative)
            mainhand.setItem(decreaseItem(item));
        inv.addItem(randomItem);
    }
    else if (item.hasTag("aquarium:jellyfish")) {
        player.addEffect("poison", 1200, { amplifier: 1 });
        player.addEffect("nausea", 300, { amplifier: 1 });
    }
});
